MODULE UTILS
USE NUMERICAL_LIBRARIES
!
IMPLICIT NONE
!
public :: CAMAT
public :: AMAT2EULER
!
CONTAINS
!
!***************************
SUBROUTINE CAMAT(EULER,AMAT)
!***************************
! Purpose: Calculates the Transformation matrix related to input Euler parameters
! Input
DOUBLE PRECISION    :: EULER(4)        ! Euler parameters
! Output
DOUBLE PRECISION    :: AMAT(3,3)       ! Transformation matrix
!
AMAT = RESHAPE((/&
         1.d0-2.d0*(EULER(3)**2+EULER(4)**2),& 
         2.d0*(EULER(2)*EULER(3)+EULER(1)*EULER(4)),&
		 2.d0*(EULER(2)*EULER(4)-EULER(1)*EULER(3)),&
         2.d0*(EULER(2)*EULER(3)-EULER(1)*EULER(4)),&
		 1.d0-2.d0*(EULER(2)**2+EULER(4)**2), &
		 2.d0*(EULER(3)*EULER(4)+EULER(1)*EULER(2)),&
		 2.d0*(EULER(2)*EULER(4)+EULER(1)*EULER(3)),&
		 2.d0*(EULER(3)*EULER(4)-EULER(1)*EULER(2)),&
		 1.d0-2.d0*(EULER(2)**2+EULER(3)**2)/),(/3,3/))
END SUBROUTINE CAMAT
!
!***************************
SUBROUTINE AMAT2EULER(A, EULER)
!***************************
! Purpose: Calculates Euler parameters ("EULER") corresponding to then Transformation Matrix ("A")
! Euler parameters defined r0,rvec (r0,r1,r2,r3)
! input
DOUBLE PRECISION :: A(3,3)
! output
DOUBLE PRECISION :: EULER(4)
real*8 :: tmp,trace,S
! locals
!
! method from http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
!tmp=A(1,1)-1.0d0
!euler(1) = dsqrt( max( 0.0d0, 1 + A(1,1) + A(2,2) + A(3,3) ) ) / 2.0d0 
!euler(2) = dsqrt( max( 0.0d0, 1 + A(1,1) - A(2,2) - A(3,3) ) ) / 2.0d0 
!euler(3) = dsqrt( max( 0.0d0, 1 - A(1,1) + A(2,2) - A(3,3) ) ) / 2.0d0 
!euler(4) = dsqrt( max( 0.0d0, 1 - A(1,1) - A(2,2) + A(3,3) ) ) / 2.0d0 
!euler(2) = dsign( euler(2), A(3,2) - A(2,3) ) 
!euler(3) = dsign( euler(3), A(1,3) - A(3,1) ) 
!euler(4) = dsign( euler(4), A(2,1) - A(1,2) ) 

! other method from http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
trace=A(1,1)+A(2,2)+A(3,3)
if (trace>0.d0) then
  S=dsqrt(trace+1.d0)*2.d0 
  euler(1)=0.25d0*S
  euler(2)=(A(3,2)-A(2,3))/S
  euler(3)=(A(1,3)-A(3,1))/S
  euler(4)=(A(2,1)-A(1,2))/S
else if ((A(1,1).GT.A(2,2)).AND.(A(1,1).GT.A(3,3))) then
  S=dsqrt(1.d0+A(1,1)-A(2,2)-A(3,3))*2.d0
  euler(1)=(A(3,2)-A(2,3))/S
  euler(2)=0.25d0*S
  euler(3)=(A(1,2)+A(2,1))/S
  euler(4)=(A(1,3)+A(3,1))/S
else if (A(2,2).GT.A(3,3)) then
  S=dsqrt(1.d0-A(1,1)+A(2,2)-A(3,3))*2.d0
  euler(1)=(A(1,3)-A(3,1))/S
  euler(2)=(A(1,2)+A(2,1))/S
  euler(3)=0.25d0*S
  euler(4)=(A(2,3)+A(3,2))/S
else
  S=dsqrt(1.d0-A(1,1)-A(2,2)+A(3,3))*2.d0
  euler(1)=(A(2,1)-A(1,2))/S
  euler(2)=(A(1,3)+A(3,1))/S
  euler(3)=(A(2,3)+A(3,2))/S
  euler(4)=0.25d0*S
endif


END SUBROUTINE
END MODULE