Module genout_tools
use genout_typesnvars

implicit none

!--------------------------------------------------------------------
!--------------------------------------------------------------------
! Public/private declarations

public :: readline
public :: getwords
public :: getwords2
public :: get_parameters

private :: lowercase
!--------------------------------------------------------------------
!--------------------------------------------------------------------
! Type definitions

!--------------------------------------------------------------------

contains

!--------------------------------------------------------------------
!--------------------------------------------------------------------

!--------------------------------------------------------------------
!--------------------------------------------------------------------
!
subroutine readline(masterfile,line,init)
Type (Tmasterfile),pointer    :: masterfile
Type (Tline),intent(inout)    :: line
logical,intent(inout)         :: init


if (init==.true.) then
  masterfile%linenr=0
  init=.false.
endif

read(unit=masterfile%unit,fmt='(A)',end=99) line
masterfile%linenr=masterfile%linenr+1

call lowercase(line)

return
99 continue
write(0,*) '*** INFO *** End of file reached. Line number',masterfile%linenr,' Masterfile name: ',trim(masterfile%name)
line%line='exit ;'
end subroutine readline

!--------------------------------------------------------------------
!--------------------------------------------------------------------

subroutine getwords(line,words,masterfile)
use dfport
! divides a line into several words spaced by zero, line ended with ;
character*(*),intent(inout)     :: line
Type (Tword), intent(inout)     :: words
Type (Tmasterfile),pointer      :: masterfile

character :: linestop,blank,tab,comma
parameter (linestop=';')
parameter (blank   =' ')
parameter (tab     ='	')
parameter (comma=',')

integer :: first, &
           last, &
		   endp, &
		   blankpos, &
		   tabpos, &
		   commapos

integer :: tmp_blank,tmp_tab

logical :: out

integer :: error, info_array(12)
logical :: init,cont
Type (Tline) :: line2

cont=.false.
DO while (cont==.false.)
cont=.true.

init=.false.

words%word_vec(:)=''

line=adjustl(line)

endp=index(line,linestop)
first=1
words%nwords=0
out=.false.

if (endp>1) then
  do while (out==.false.)
    blankpos=index(line(first:endp),blank)
	tabpos  =index(line(first:endp),tab)
	commapos=index(line(first:endp),comma)
	if (commapos/=0) write(0,*) '*** WARNING *** A comma "," is written within the command line. File ',trim(masterfile%name),' line nr.',masterfile%linenr

	if (tabpos/=0) then
	  if (blankpos/=0) then
	    blankpos=min(blankpos,tabpos)
	  else 
	    blankpos=tabpos
	  endif
	endif
!   If the restline is without blanks, the restline is stored and
!   the division in words is stopped ...
    if (blankpos==0) then
      words%nwords=words%nwords+1
	  if (words%nwords>size(words%word_vec)) goto 20
      words%word_vec(words%nwords)=line(first:endp-1)
	  out=.true.
      exit
    endif

!   If a blank is found a word is stored ...
    if (blankpos>1) then
      words%nwords=words%nwords+1
	  if (words%nwords>size(words%word_vec)) goto 20
      last=first+blankpos-2
      words%word_vec(words%nwords)=line(first:last)
      first=first+blankpos
    endif

!   If search has reached and end of line or if too many words have
!   been found then stop
    if (first>=endp) then
	  out=.true.
	  exit
	endif
	if (words%nwords>size(words%word_vec)) goto 20

!   else remove blanks in restline by incrementing the first position
    tmp_blank=index(line(first:endp),blank)
	tmp_tab=index(line(first:endp),tab)
	if (tmp_tab/=0) then
	  if (tmp_blank/=0) then
	    tmp_blank=min(tmp_blank,tmp_tab)
	  else 
	    tmp_blank=tmp_tab
	  endif
	endif

!	if (tmp_tab/=0) tmp_blank=min(tmp_blank,tmp_tab)
    do while (tmp_blank==1)
      first=first+1
	  if (first>=endp) then
	    out=.true.
  	    exit
	  endif
      tmp_blank=index(line(first:endp),blank)
	  tmp_tab=index(line(first:endp),tab)
	  if (tmp_tab/=0) tmp_blank=min(tmp_blank,tmp_tab)
    enddo
  enddo
elseif (endp==0) then 
  write(0,*) '*** ERROR *** No line termination in command line ',masterfile%linenr
endif

ENDDO

return
  10 write(0,*) 'Error opening file: ',trim(masterfile%name)
     stop
  20  write(0,*) '*** ERROR *** More than ',size(words%word_vec),' words in line. Exceeding words ignored'
      write(0,*) ' Line content: ',line
end subroutine getwords

!--------------------------------------------------------------------
!--------------------------------------------------------------------

subroutine getwords2(line,words)
! As getwords, but no line end used
character*(*),intent(inout)     :: line
Type (Tword), intent(inout)     :: words

character :: blank,tab
parameter (blank   =' ')
parameter (tab     ='	')

integer :: first, &
           last, &
		   endp, &
		   blankpos, &
		   tabpos

integer :: tmp_blank,tmp_tab

logical :: out

words%word_vec(:)=''

line=adjustl(line)

endp=len(line)
first=1
words%nwords=0
out=.false.

if (endp>1) then
  do while (out==.false.)
    blankpos=index(line(first:endp),blank)
	tabpos=index(line(first:endp),tab)
	if (tabpos/=0) then
	  if (blankpos/=0) then
	    blankpos=min(blankpos,tabpos)
	  else 
	    blankpos=tabpos
	  endif
	endif
!   If the restline is without blanks, the restline is stored and
!   the division in words is stopped ...
    if (blankpos==0) then
      words%nwords=words%nwords+1
      if (words%nwords>size(words%word_vec)) goto 10
      words%word_vec(words%nwords)=line(first:endp-1)
	  out=.true.
      exit
    endif

!   If a blank is found a word is stored ...
    if (blankpos>1) then
      words%nwords=words%nwords+1
      if (words%nwords>size(words%word_vec)) goto 10
      last=first+blankpos-2
      words%word_vec(words%nwords)=line(first:last)
      first=first+blankpos
    endif

!   If search has reached and end of line or if too many words have
!   been found then stop
    if (first>=endp) then
	  out=.true.
	  exit
	endif
	if (words%nwords>size(words%word_vec)) goto 10

!   else remove blanks in restline by incrementing the first position
    tmp_blank=index(line(first:endp),blank)
	tmp_tab=index(line(first:endp),tab)
	if (tmp_tab/=0) then
	  if (tmp_blank/=0) then
	    tmp_blank=min(tmp_blank,tmp_tab)
	  else 
	    tmp_blank=tmp_tab
	  endif
	endif
!	if (tmp_tab/=0) tmp_blank=min(tmp_blank,tmp_tab)
    do while (tmp_blank==1)
      first=first+1
	  if (first>=endp) then
	    out=.true.
  	    exit
	  endif
      tmp_blank=index(line(first:endp),blank)
	  tmp_tab=index(line(first:endp),tab)
	  if (tmp_tab/=0) tmp_blank=min(tmp_blank,tmp_tab)
    enddo
  enddo
endif

return

10  continue ! Exceeding words do not produce an error message


end subroutine getwords2

!--------------------------------------------------------------------
!--------------------------------------------------------------------

subroutine get_parameters(words,nstart,npars,out_vec,err)
Type (Tword), intent(in)            :: words
integer , intent(in)                :: nstart, &
                                       npars
real*8, dimension(:), intent(inout) :: out_vec
logical, intent(inout)              :: err

integer :: i

if (words%nwords<(nstart+npars-1)) then
  write(0,*) '*** ERROR *** Not correct number of parameters in command line'
  err=.true.
  goto 99
else
  do i=nstart,nstart+npars-1
    read(words%word_vec(i),Fmt=*,err=99) out_vec(i-nstart+1)
    err=.false.
  enddo
endif
return
99 write(0,*) '*** ERROR *** Error reading parameters in command line: ',trim(words%word_vec(1)),' ',trim(words%word_vec(2))
   err=.true.
end subroutine get_parameters

!--------------------------------------------------------------------
subroutine get_label(words,label)
! This subroutine searches the input words for a label which should be enclosed in #'s
Type (Tword),intent(in)              :: words
character*(*),intent(out)            :: label

logical :: mark1, &
           mark2
integer   :: i

label=''
mark1=.false.
mark2=.false.
do i=1,size(words%word_vec)
  if (mark1==.false.) then
    if (trim(words%word_vec(i))=='#') then
      mark1=.true.
	endif
  else
    if (trim(words%word_vec(i))=='#') then
      mark2=.true.
	  label=trim(label)//'#'
	else
      label=trim(label)//' '//trim(words%word_vec(i))
	endif
  endif
  if (mark2==.true.) then
    cycle
  endif
enddo
end subroutine get_label
!--------------------------------------------------------------------

!--------------------------------------------------------------------
!--------------------------------------------------------------------
!--------------------------------------------------------------------

Function onoff(word,masterfile)
character*wordlenght, intent(in) :: word
Type (Tmasterfile), pointer      :: masterfile
character*3                      :: onoff

onoff=trim(word)
if (onoff=='on') then
  continue
elseif (onoff=='off') then
  continue
else
  write(0,*) '*** ERROR *** Error in regarding on/off parameter in line ',masterfile%linenr
endif

end function onoff
!--------------------------------------------------------------------
!--------------------------------------------------------------------

subroutine lowercase(line)
Type (Tline), intent(inout) :: line

integer   :: i
logical   :: marker

marker=.false.
do i=1,len(line%line)
  if ((line%line(i:i).eq."'").and.(marker.eq..false.)) then
    marker=.true.
	line%line(i:i)=' '
  else if ((line%line(i:i).eq."'").and.(marker.eq..true.)) then
    marker=.false.
	line%line(i:i)=' '
  else if ((line%line(i:i).ge.'A').and.(line%line(i:i).le.'Z').and.(marker.eq..false.)) then
    line%line(i:i)=char(ichar(line%line(i:i))+32)
  endif
enddo
end subroutine lowercase

!--------------------------------------------------------------------
!--------------------------------------------------------------------

end module genout_tools
