Module global_vars  ! types and variables used several windturb modules
implicit none

!--------------------------------------------------------------------
!--------------------------------------------------------------------

!--------------------------------------------------------------------
!--------------------------------------------------------------------
! Type declarations

! TYPES
!
TYPE TNODE
  INTEGER          :: ID = 0
  DOUBLE PRECISION :: R(3)
END TYPE
!
TYPE TMEMBER
  INTEGER                        :: ID = 0
  INTEGER                        :: NODE1, NODE2     ! Node number end 1 and 2
  DOUBLE PRECISION               :: D1,D2   ! Outer diameter end 1 and 2
  DOUBLE PRECISION               :: T1,T2   ! Wall thickness end 1 and 2
  INTEGER						 :: marine_growth=0 ! Is marine growth present (thickness 0.1m!)?
  real*8						 :: Dif		! Internal diameter for flooded part
!  DOUBLE PRECISION, DIMENSION(3) :: EX,EY,EZ ! Directional cosines of local x, y, and z axes
  real*8,dimension(3,3)			 :: TLG     ! transformation matrix from local body coo to global
!
  INTEGER                        :: HTC_DIVISION = 1 ! sub-division in htc-file
  INTEGER                        :: ST_DIVISION = 1  ! sub-division in st-file
END TYPE
!
INTEGER, PARAMETER :: MMAX = 50
!
TYPE TBODYDEF
  CHARACTER*256                   :: NAME
  INTEGER                         :: NMEMBERS = 0
  INTEGER, DIMENSION(:),pointer   :: MEMBERID=>null()     ! ID of member
  !
  LOGICAL                         :: BREORDER = .FALSE.   ! Flag indicating that member node order is reversed
  DOUBLE PRECISION                :: EULER(4)             ! Euler parameters corr. to transformation matrix (local->global (BLADED))
  DOUBLE PRECISION                :: INIPOS(3)            ! Initial position of body origin
  ! Derived variables
  INTEGER                         :: NSEC                 ! NOF sections in body (nodes)
  DOUBLE PRECISION, POINTER       :: RLOCAL(:,:)          ! ....and their local positions
END TYPE
!
Type Tfix0
  Type (Tbodydef),pointer		:: b
end type tFix0
!
Type Tfix1
  Type (Tbodydef),pointer		:: b1,b2
  Real*8						:: epsdist=0.d-5
end type Tfix1
!
DOUBLE PRECISION				:: Emod = 2.10d11
DOUBLE PRECISION				:: Gmod = 8.10d10
DOUBLE PRECISION				:: rho  = 7850.d0
!
TYPE TSTLINE
  DOUBLE PRECISION :: r,               &
                      m,               &
					  x_cg = 0.d0,     &
					  y_cg = 0.d0,     &
					  rix,             &
					  riy,             &
					  xshear = 0.d0,   &
					  yshear = 0.d0,   &
					  E = 0.d0,        &
					  G = 0.d0,        &
					  I_x,             &
					  I_y,             &
					  I_p,             &
					  k_x = 0.5d0,     &
					  k_y = 0.5d0,     &
					  Ae,              &
					  spitch_l = 0.d0, &
					  x_le_l = 0.d0,   &
					  y_le_l = 0.d0

END TYPE

!--------------------------------------------------------------------
!--------------------------------------------------------------------
! Public/private declarations

!INTEGER									:: NBODIES = 0
TYPE (TBODYDEF), DIMENSION(:),POINTER	:: BODIES=>NULL()
Type (Tfix0),dimension(:),pointer		:: fix0=>null()
Type (Tfix1),dimension(:),pointer		:: fix1=>null()


real*8				:: mwl=0.d0 !
real*8				:: mudlevel=50.d0
real*8				:: density_water=1250d0
real*8				:: gravity=9.80665d0
real*8				:: mg_rho=1100 ! Marine growth density
real*8				:: mg_thick=0.1 ! Marine growth thickness


TYPE (TNODE),   DIMENSION(:), pointer	:: NODES
TYPE (TMEMBER), DIMENSION(:), pointer	:: MEMBERS
!DOUBLE PRECISION, PARAMETER, DIMENSION(3,3) :: TBH_GLOBAL = (/ 0.d0, 1.d0, 0.d0, &
!                                                               1.d0, 0.d0, 0.d0, &
!													           0.d0, 0.d0,-1.d0/)
!test tjul
DOUBLE PRECISION, DIMENSION(3,3) :: TBH_GLOBAL = (/ 1.d0, 0.d0, 0.d0, &
                                                               0.d0, 1.d0, 0.d0, &
													           0.d0, 0.d0, 1.d0/)



!--------------------------------------------------------------------
!--------------------------------------------------------------------

end module global_vars