PROGRAM MAIN
!
USE GENINPUT
use global_vars
use genout_tools2
!
IMPLICIT NONE
!
Type (TBODYDEF),pointer					:: b=>null()
Type (TMEMBER),pointer					:: m=>null()
Type (Tfix0),pointer					:: f0=>null()
Type (Tfix1),pointer					:: f1=>null()

!INTEGER, PARAMETER :: stdoutunit = 6
INTEGER            :: unit
INTEGER            :: i,n,j,q
CHARACTER*255      :: filename
LOGICAL			   :: found
!
write (0,*) "HAWC2 JACKET PREPROCESSOR"
write (0,*) "VERSION 1.1"
write (0,*) "DTU, WIND ENERGY"
write (0,*) "www.hawc2.dk"
write (0,*) "hawc2@risoe.dtu.dk"
write (0,*)
CALL getfreeunit(unit)

CALL GENINPUT_READCONNECT("connections.txt")

CALL GENINPUT_READNODES("nodes.txt")
CALL GENINPUT_READMEMBERS("members.txt")
CALL get_orientations()
CALL GENINPUT_2HAWC2()
!
! Replace presetn memberid with actual member number in members(:)
do n=1,size(bodies)
  !find node related to member nodeid
  b=>bodies(n)
  do i=1,b%nmembers
    found=.false.
    do j=1,size(members)
	  if (b%memberid(i).eq.members(j)%id) then
	    b%memberid(i)=j
	    found=.true.
	    exit
	  endif
	enddo
  enddo
enddo
!
!! Sub-divide conical members
!MEMBERS(46)%HTC_DIVISION =  1
!MEMBERS(46)%ST_DIVISION  =  2
!MEMBERS(47)%HTC_DIVISION = 5
!MEMBERS(47)%ST_DIVISION  = 10
!DO i=59,73
!  MEMBERS(i)%HTC_DIVISION = 1
!  MEMBERS(i)%ST_DIVISION  = 3
!ENDDO
!DO i=40,45
!  MEMBERS(i)%HTC_DIVISION = 2
!  MEMBERS(i)%ST_DIVISION  = 2
!ENDDO
!DO i=13,18
!  MEMBERS(i)%HTC_DIVISION = 2
!  MEMBERS(i)%ST_DIVISION  = 2
!ENDDO
!
!*******************
! Check bodies
!*******************
!DO i=1,NBODIES
!  CALL GENINPUT_CHECK(bodies(i))
!ENDDO
!*******************
! Create bodies
!*******************
call getfreeunit(unit)
OPEN(unit,FILE="bodies.dat")
DO i=1,size(bodies)
  CALL GENINPUT_HTC(bodies(i),unit)
ENDDO
WRITE(unit,'("exit ;")')
WRITE(unit,'(";")')
CLOSE(unit)
!*******************
! Create body orientations
!*******************
OPEN(unit,FILE="orientation.dat")
!
DO i=1,size(bodies)
  CALL GENINPUT_ORIENTATION(bodies(i),unit)
ENDDO
!
WRITE(unit,'("exit ;")')
WRITE(unit,'(";")')
CLOSE(unit)
!*******************
! Create constraints
!*******************
OPEN(unit,FILE="constraints.dat")
!
! Feet
!CALL GENINPUT_FIX0(F1,unit)
do i=1,size(fix0)
  f0=>fix0(i)
  CALL GENINPUT_FIX0(f0%b,unit)
enddo

!
! Connections to L1
!CALL GENINPUT_FIX1(L1,F1,unit)
do i=1,size(fix1)
  f1=>fix1(i)
  CALL GENINPUT_FIX1(f1%b1,f1%b2,f1%epsdist,unit)
enddo
!
WRITE(unit,'("exit ;")')
WRITE(unit,'(";")')
CLOSE(unit)
!*******************
! Create st-files
!*******************
DO i=1,size(bodies)
  WRITE(filename,'("data/",A,".stc")') bodies(i)%NAME(1:LEN_TRIM(bodies(i)%NAME))
  OPEN(unit,FILE=filename)
  CALL GENINPUT_ST(bodies(i),unit)
  CLOSE(unit)
ENDDO
!
!*******************
! Create hydro elements
!*******************
OPEN(unit,FILE="hydro.dat")
WRITE(unit,'(''begin HYDRO ;'')')
WRITE(unit,'(''  begin WATER_PROPERTIES ;'')')
WRITE(unit,fmt='(A, 1pe12.5, A)')"    rho      ",density_water," ; [kg/m^3]"
WRITE(unit,fmt='(A, 1pe12.5, A)')"    gravity  ",gravity," ; [m/s^2]"
WRITE(unit,fmt='(A, 1pe12.5, A)')"    mwl      ",mwl," ; [m]"
WRITE(unit,fmt='(A, 1pe12.5, A)')"    mudlevel ",mudlevel," ; [m]"
WRITE(unit,'(''    water_kinematics_dll ./wkin_dll.dll ./hydrohtc/reg_airy_0.htc ;'')')
WRITE(unit,'(''  end WATER_PROPERTIES ;'')')
WRITE(unit,'('';'')')
DO i=1,size(bodies)
  CALL GENINPUT_HYDRO(bodies(i),unit)
ENDDO
WRITE(unit,'("END HYDRO ;")')
WRITE(unit,*)"exit ;"
WRITE(unit,'(";")')
CLOSE(unit)
!
!*******************
! Create OUTPUT COMMANDS
!*******************
OPEN(unit,FILE="output_global.dat")
WRITE(unit,'("begin output ;")')
WRITE(unit,'("  filename ./substructure_out_glcoo ;")')
WRITE(unit,'("  data_format hawc_binary ;")')
WRITE(unit,'("  ;")')
WRITE(unit,'("  general time ;")')
DO i=1,size(bodies)
  b=>bodies(i)
  do q=1,b%nmembers
    m => MEMBERS(b%MEMBERID(q))
    WRITE(unit,fmt='(A, A, I3, A,A,I5,A)')"  mbdy state pos  ",trim(b%name),q,"  0.0 global # ","node nr ",m%node1," ;"
  enddo
  m => MEMBERS(b%MEMBERID(b%nmembers))
  WRITE(unit,fmt='(A, A, I3, A,A,I5,A)')"  mbdy state pos  ",trim(b%name),b%nmembers,"  1.0 global # ","node nr ",m%node2," ;"
enddo
WRITE(unit,'("  ;")')
!
DO i=1,size(bodies)
  b=>bodies(i)
  do q=1,b%nmembers
    m => MEMBERS(b%MEMBERID(q))
    WRITE(unit,fmt='(A, A, I3, A,A,I5,A,I5,A)')"  mbdy forcevec   ",trim(b%name),q,"  0.0 global # ","member nr",m%id," node nr ",m%node1," ;"
    WRITE(unit,fmt='(A, A, I3, A,A,I5,A,I5,A)')"  mbdy momentvec  ",trim(b%name),q,"  0.0 global # ","member nr",m%id," node nr ",m%node1," ;"
  enddo
  m => MEMBERS(b%MEMBERID(b%nmembers))
  WRITE(unit,fmt='(A, A, I3, A,A,I5,A,I5,A)')"  mbdy forcevec   ",trim(b%name),b%nmembers,"  1.0 global # ","member nr",m%id," node nr ",m%node2," ;"
  WRITE(unit,fmt='(A, A, I3, A,A,I5,A,I5,A)')"  mbdy momentvec  ",trim(b%name),b%nmembers,"  1.0 global # ","member nr",m%id," node nr ",m%node2," ;"
enddo
WRITE(unit,'("  ;")')
WRITE(unit,'("end output ;")')

WRITE(unit,*)"exit ;"
WRITE(unit,'(";")')
CLOSE(unit)
!
OPEN(unit,FILE="output_body.dat")
WRITE(unit,'("begin output ;")')
WRITE(unit,'("  filename ./substructure_out_bodycoo ;")')
WRITE(unit,'("  data_format hawc_binary ;")')
WRITE(unit,'("  ;")')
WRITE(unit,'("  general time ;")')
DO i=1,size(bodies)
  b=>bodies(i)
  do q=1,b%nmembers
    m => MEMBERS(b%MEMBERID(q))
    WRITE(unit,fmt='(A, A, I3, 4(A),I5,A,I5,A)')"  mbdy forcevec   ",trim(b%name),q,"  0.0 ",trim(b%name)," # ","member nr",m%id," node nr ",m%node1," ;"
    WRITE(unit,fmt='(A, A, I3, 4(A),I5,A,I5,A)')"  mbdy momentvec  ",trim(b%name),q,"  0.0 ",trim(b%name)," # ","member nr",m%id," node nr ",m%node1," ;"
  enddo
  m => MEMBERS(b%MEMBERID(b%nmembers))
  WRITE(unit,fmt='(A, A, I3, 4(A),I5,A,I5,A)')"  mbdy forcevec   ",trim(b%name),b%nmembers,"  1.0 ",trim(b%name)," # ","member nr",m%id," node nr ",m%node2," ;"
  WRITE(unit,fmt='(A, A, I3, 4(A),I5,A,I5,A)')"  mbdy momentvec  ",trim(b%name),b%nmembers,"  1.0 ",trim(b%name)," # ","member nr",m%id," node nr ",m%node2," ;"
enddo
WRITE(unit,'("  ;")')
WRITE(unit,'("end output ;")')

WRITE(unit,*)"exit ;"
WRITE(unit,'(";")')
CLOSE(unit)
!
write(0,*) "Hit return to end"
read(*,*) 

END PROGRAM